<?php

namespace Twine\forms\strategies\layout;

use Twine\forms\base\FormSectionBase;
use Twine\forms\base\FormSectionDetails;
use Twine\forms\base\FormSection;
use Twine\forms\inputs\FormInputBase;
use Twine\helpers\Html;

class DetailsSummaryLayout extends FormSectionLayoutBase
{
    /**
     * @var FormSectionBase
     */
    protected $inner_layout;

    /**
     * DetailsSummaryLayout constructor.
     *
     * @param FormSectionLayoutBase $inner_layout
     */
    public function __construct(FormSectionLayoutBase $inner_layout)
    {
        $this->inner_layout = $inner_layout;
        parent::__construct();
    }

    public function constructFinalize(FormSection $form)
    {
        $this->inner_layout->constructFinalize($form);
        parent::constructFinalize($form);
    }

    public function layoutFormBegin()
    {
        $html_generator = Html::instance();
        if ($this->Form_section instanceof FormSectionDetails) {
            $summary = $this->Form_section->getSummary();
        } else {
            $summary = __('Show Options', 'print-my-blog');
        }

        return $this->displayFormWideErrors()
               . $html_generator->openTag(
                   'details',
                   $this->Form_section->htmlId(),
                   $this->Form_section->htmlClass() . ' twine-details',
                   $this->Form_section->htmlStyle()
               ) . $html_generator->tag(
                   'summary',
                   $summary,
                   $this->Form_section->htmlId() . '-summary',
                   'twine-summary'
               ) . $this->inner_layout->layoutFormBegin() . $this->inner_layout->layoutFormLoop();
    }

    public function layoutForm()
    {
        return parent::layoutForm(); // TODO: Change the autogenerated stub
    }

    public function layoutFormEnd()
    {
        $html_generator = Html::instance();
        return $this->inner_layout->layoutFormEnd() . $html_generator->closeTag('details');
    }

    public function layoutInput($input)
    {
        $this->inner_layout->layoutInput($input);
    }

    public function layoutSubsection($subsection)
    {
        $this->inner_layout->layoutSubsection($subsection);
    }
}
